function [bestSize, minErr] = optimizeNeighborhoodLocalSda( ...
  neighborhoodSizeVec, simMat, classLabels)
%optimizeNeighborhoodLocalSda
% Optimize the neighborhood size for the local SDA classifier,using the
% cross-validation strategy decribed by Hastie, Tibshirani and % Friedman
% (HTF) in "The Elements of Statistical Learning", pages 214-216.
% Assumes similarity data are used (not dissimilarity) and the centroid
% type is 'maxSumSim'.
%
% USAGE
% [bestSize, minErr] = optimizeNeighborhoodLocalSda(neighborhoodSizeVec, ...
%     simMat, classLabels)
% neighbrohoodSizeVec: vector of neighborhood size to cross-validate.
% simMat: N-by-N matrix of pairwise similarities.
% classLabels: N-by-1 vector of class albesl for each row of simMat.
% bestSize: optimized neighborhood size.
% minErr: cross-validation error corresponding to best neighborhood size.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, Feb 2007.
%$Id:$
%$DateTime:$
%$Author:$

neighborhoodSizeVec = unique(neighborhoodSizeVec);
[nData, notUsed] = size(simMat); %#ok
maxNeighbors = min( [max(neighborhoodSizeVec) nData]);
neighborhoodSizeVec = [neighborhoodSizeVec(:).' maxNeighbors];
neighborhoodSizeVec(neighborhoodSizeVec > (maxNeighbors - 1) ) = [];
neighborhoodSizeVec = unique(neighborhoodSizeVec);
nNeighborhoodSizes = length(neighborhoodSizeVec);

neighborIdx = findNearNeighbors(simMat, maxNeighbors, 'similarity');
neighborIdx = adjustSelfSimilarityRank(neighborIdx);
neighborIdx(:,1) = [];

errVec = 10e10 * ones(nNeighborhoodSizes,1); 
for iSize = 1:nNeighborhoodSizes
  thisSize = neighborhoodSizeVec(iSize);
  
  classEst = 10e10 * ones(nData,1);
  for iData = 1:nData
    try
    neighborsToPick = neighborIdx(iData,1:thisSize);
    catch
      keyboard
    end
    neighborClassLabels = classLabels(neighborsToPick);
    testPointSim = simMat( iData, neighborsToPick );
    neighborSimMat = simMat( :, neighborsToPick );
    neighborSimMat = neighborSimMat( neighborsToPick, : );
    
    classEst(iData) = classifyLocalSda(testPointSim, neighborSimMat, ...
      neighborClassLabels, 'maxSumSim');
  end %for iData  
  
  errVec(iSize) = sum( classEst(:) ~= classLabels(:) );   
end %for iSize
 
[minErr, minErrIdx] = min(errVec);
bestSize = neighborhoodSizeVec(minErrIdx);

return