function [bestSize, bestRegParam, minErr] = ...
  crossValidateParamsRegularizedMeanLocalSda( neighborhoodSizeVec, neighborType, ...
  regParamVec, simMat, crossValIdxMat, classLabels, simDomain, testIdx, meanType)
%crossValidateParamsRegularizedMeanLocalSda - cross validate parameters.
% Cross-validate the neighborhood size and the regularization parameter
% for the local SDA classifier regularized by the mean similarity.
% Uses the N-fold cross validation strategy. This function is designed to be
% used in conjunction with readPartitionFile() and with the partition files
% from the Similarity Learning Repository.
%
% Assumes similarity data are used (not dissimilarity) and the centroid
% type is 'maxSumSim'.
%
% USAGE
% [bestSize, bestRegParam, minErr] = crossValidateParamsRegularizedMeanLocalSda( ...
%     neighborhoodSizeVec, neighborType, regParamVec, simMat, ...
%     crossValIdxMat, classLabels, simDomain, testIdx, lambdaType)
% neighbrohoodSizeVec: vector of neighborhood sizes to cross-validate.
% neighborType: string, can be either 'kPerClass' or 'kTotal'.
% regParamVec: vector of regularization parameters to cross-validate.
% simMat: N-by-N matrix of pairwise similarities.
% crossValIdxMat: N-by-K matrix of indices into the rows of simMat,
%    denoting the samples to be used for cross-validation. It is a cell
%    from crossValIdxCell as returned by readPartitionFile().
% classLabels: N-by-1 vector of class labels for each row of simMat.
% simDomain: vector of possible similarity values.
% testIdx: a vector of indices into the columns of simMat denoting which
%     columns should be reserved for the testing data set. It is a row from
%     testIdxMat as returned by readPartitionFile(), corresponding to
%     the matrix crossValIdxMat.
% meanType: string, either 'average' or 'global'.
% bestSize: optimized neighborhood size.
% bestRegParam: optimized regularization parameter.
% minErr: cross-validation error rate corresponding to the optimized
%     parameters. 

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, March 2009.
%$Id:$
%$DateTime:$
%$Author:$

%% Preliminaries

% Specify similarity domain
if isempty(simDomain)
  simDomain = unique(simMat(:));
else
  simDomain = unique(simDomain);
end

% Vector of regularizing parameter values to test
regParamVec = unique(regParamVec);
nRegParams = length(regParamVec);

% Number of cross-validation folds
nFolds = size(crossValIdxMat,1);

% END of preliminary stuff

%% Pre-compute the regularizing mean similarities for each fold
regMeanStorage = cell(nFolds, 1);
regClassStorage = cell(nFolds, 1);

% Global or average mean?
switch lower(meanType) 
  % Regularize by the global SDA means
  case 'global' 
  for iFold = 1:nFolds
    disp(['Pre-computing global SDA model parameters for fold: ' num2str(iFold)])
    
    % Get the cross-validation - that is, the "test" sample for this fold
    crossValIdx = crossValIdxMat(iFold,:);
    crossValIdx(crossValIdx == 0) = []; % take out any remaining filler values

    % Compute the global SDA distribution using the rest of the
    % samples as training samples for this cross-val fold ...

    % ... take out the true test samples ...
    thisFoldTrainIdx = 1:size(simMat,2);
    thisFoldTrainIdx(testIdx) = [];

    % ... take this fold's "test" points out of this fold's training data ...
    nCrossVals = length(crossValIdx);
    for iCrossVal = 1:nCrossVals
      idx = find(thisFoldTrainIdx == crossValIdx(iCrossVal));
      thisFoldTrainIdx(idx) = [];
    end
    thisFoldTrainSimMat = simMat(:, thisFoldTrainIdx);
    thisFoldTrainSimMat = thisFoldTrainSimMat(thisFoldTrainIdx,:);
    thisFoldTrainClassLabels = classLabels(thisFoldTrainIdx);

    thisFoldGlobalSdaModelParams = computeMaxEntModels( ...
      thisFoldTrainSimMat, thisFoldTrainClassLabels, 'maxSumSim', simDomain);
    regMeanStorage{iFold} = thisFoldGlobalSdaModelParams.meanSim;
    regClassStorage{iFold} = thisFoldGlobalSdaModelParams.classLabels;
  end % for iFold
  
  case 'average'
    for iFold = 1:nFolds
      regLambdaStorage{iFold} = [];
      regClassStorage{iFold} = [];
    end
    
  otherwise
    error('Unknown regularization type');
end % switch
      

%% Cross validate  

% Number of different neighborhoods to try
actualNeighborhoodSizeVec = unique(neighborhoodSizeVec);
nSizes = length(actualNeighborhoodSizeVec);


%% Cross-validation loops
errMat = 0;
    
iterCount = 0;
for iFold = 1:nFolds 
  
  disp(['Cross-validation fold: ' num2str(iFold)]);
 
  % The regularizing lambdas to use for this fold
  regMean = regMeanStorage{iFold};
  regClassLabels = regClassStorage{iFold}; 
  
  % Get the cross-validation - that is, the "test" sample for this fold
  crossValIdx = crossValIdxMat(iFold,:);
  crossValIdx(crossValIdx == 0) = []; % take out any remaining filler values

  % Iterate through all the "test" samples in this cross-validation fold
  nCrossVals = length(crossValIdx);
  for iCrossVal = 1:nCrossVals

    % get the ith "test" sample's index
    thisCrossValSampleIdx = crossValIdx(iCrossVal);
    thisCrossValClass = classLabels(thisCrossValSampleIdx);

    % Remove the sample itself from candidate neighbors indeces.
    candidateNeighborIdx = 1:size(simMat,2);
    candidateNeighborIdx(thisCrossValSampleIdx) = [];
    
    % Remove other cross validation "test" points as candidate neighbors
    % by looking through each of them and removing them
    for iCount = 1:nCrossVals
      idx = find( candidateNeighborIdx == crossValIdx(iCount) );
      candidateNeighborIdx(idx) = [];
    end % for iCount

    % Remove the true test samples as candidate neighbors
    % by looking through each of them and removing them
    for iTest = 1:length(testIdx)
      idx = find( candidateNeighborIdx == testIdx(iTest) );
      candidateNeighborIdx(idx) = [];
    end % for iTest

    % Compose matrix of candidate neighbor similarities
    candidateNeighborClassLabels = classLabels(candidateNeighborIdx);
    candidateNeighborSimMat = simMat( :, candidateNeighborIdx );
    candidateNeighborSimMat = candidateNeighborSimMat( candidateNeighborIdx, : );
    testPointSimToCandidates = simMat( thisCrossValSampleIdx, candidateNeighborIdx );
   
    % Get the actual neighbors
    if strcmpi(neighborType, 'kPerClass')
      neighborIdxByClass = findNearNeighborsByClass(testPointSimToCandidates, ...
        size(candidateNeighborSimMat,2), candidateNeighborClassLabels, 'similarity');
          % There should be only one element in the cell array, a matrix.
          neighborIdxMat = neighborIdxByClass{1};
    elseif strcmpi(neighborType, 'kTotal')
      neighborIdxMat = findNearNeighbors(testPointSimToCandidates, ...
        size(candidateNeighborSimMat,2));
    else
      error(['Unsupported neighborhood type: ' neighborType]);
    end
      
    % Iterate over the neighborhood sizes
    classEstMat = -1e6*ones(nSizes, nRegParams);

    for iSize = 1:nSizes
      
      nNeighbors = actualNeighborhoodSizeVec(iSize);
      actualNeighborIdx = neighborIdxMat(:, 1:nNeighbors);
      actualNeighborIdx = actualNeighborIdx(:).';

      actualTestPointSim = testPointSimToCandidates(actualNeighborIdx);
      actualNeighborSimMat = candidateNeighborSimMat(:, actualNeighborIdx);
      actualNeighborSimMat = actualNeighborSimMat(actualNeighborIdx, :);
      actualNeighborClassLabels = candidateNeighborClassLabels(actualNeighborIdx);

      % Iterate over the possible values of the reg parameter
      classEst =  -1e6 * ones(1, nRegParams);
      for iParam = 1:nRegParams
        regularizingParams = createRegularizedMeanParamStruct( ...
          meanType, regMean, regParamVec(iParam), neighborType, ...
          'maxSumSim', nNeighbors, simDomain, regClassLabels);
                       
        classEst(iParam) = classifyRegularizedMeanLocalSda(...
          actualTestPointSim, actualNeighborSimMat, ...
          actualNeighborClassLabels, regularizingParams);
 
      end % for iParam

      classEstMat(iSize,:) = classEst;
    
    end %for iSize
    
    
    % Accumulate the count of incorrect classifications for all samples in
    % all cross validation folds
    thisCrossValErrMat = double(classEstMat ~= thisCrossValClass);
    errMat = errMat + thisCrossValErrMat;
    
    iterCount = iterCount + 1;
    
  end % for iCrossVal

end % for iFold

avgErrMat = errMat / iterCount;

% Get the parameter values corresponding to the minimum error
[bestSize, bestRegParam, minErr] = local_findBestParams(avgErrMat, ...
  actualNeighborhoodSizeVec, regParamVec);

return % end main function

%% LOCAL FUNCTIONS
%local_findBestParams()
function [bestSize, bestRegParam, minErr] = ...
  local_findBestParams( errMat, neighborhoodSizeVec, regParamVec)

[minErrTemp, minErrSizeIdx] = min(errMat);
[minErr, minErrRegParamIdx] = min(minErrTemp);
bestRegParam = regParamVec(minErrRegParamIdx);

bestSize = neighborhoodSizeVec(minErrSizeIdx(minErrRegParamIdx));

return % END local_findBestParas()
