function [errCount, classEst, posteriors, nonRegModelParams] = ... 
  evaluateTestsAverageLocalSda( regularizingParams, simMat, testIdx, ...
  classLabels)
%evaluateTestsAverageLocalSda
% Compute the local SDA classification results for the test splits, using
% the average local SDA regularization strategy.
% This function is designed to be used in conjunction with
% readPartitionFile() and with the partition files from the Similarity
% Learning Repository.
%
% Assumes similarity data are used (not dissimilarity) and the centroid
% type is 'maxSumSim'.
%
% USAGE
% [errorCount, classEst, posteriors] = evaluateTestsAverageLocalSda( ...
%     regularizingParams, simMat, testIdx, classLabels, simDomain)
% regularizingParams: the parameters to use, as returned by
%    createRegularizingParamStruct().
% simMat: N-by-N matrix of pairwise similarities.
% testIdx: vector of indices into simMat denoting the samples to be tested.
%    It is a row from testIdxMat as returned by readPartitionFile().
% classLabels: N-by-1 vector of class labels for each column of simMat.
% errorCount: number of misclassified test samples.
% classEst: M-by-1 cell array; each element contains a structure as
%    returned by classifyRegularizedLocalSda(). 
% posteriors:  M-by-1 cell array; each element contains a structure as
%    returned by classifyRegularizedLocalSda(). 
% nonRegModelParams: M-by-1 cell array; each element contains a structure as
%    returned by classifyRegularizedLocalSda(). 
%
% See also crossValidateNeighborhoodLocalSda, readPartitionFile, 
%    createRegularizingParamStruct, classifyRegularizedLocalSda.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, Jan 2009.
%$Id:$
%$DateTime:$
%$Author:$

nTests = length(testIdx);
classEst = cell(nTests,1);
posteriors = cell(nTests,1);
nonRegModelParams = cell(nTests, 1);
trueClass = classLabels(testIdx);
errCount.avgMarginal = 0;
errCount.avgMarginalUniformPriors = 0;
errCount.avgJoint = 0; 
errCount.avgJointUniformPriors = 0;
errCount.avgSimToCentroids = 0;
errCount.avgAcrossClasses = 0;

%% PUT THIS HERE TO FIX BUG
% First pass at defining neighborhood: get as many neighbors from each
  % class as possible, making sure the self-similar sample is taken out of
  % the pool of candidate neighbors.
  if strcmpi(regularizingParams.neighborType, 'kPerClass')
    neighborIdxPrelim = findNearNeighborsByClass(simMat, ...
      size(simMat,2), classLabels, 'similarity');
    neighborIdxPrelim = adjustSelfSimilarityRankByClass(neighborIdxPrelim, ...
      classLabels, 'remove');
      
    % Pull out the test sample's neighbors
    candidateNeighbors = neighborIdxPrelim{1};
    candidateNeighbors = candidateNeighbors(:);
  elseif strcmpi(regularizingParams.neighborType, 'kTotal')
    neighborIdxPrelim = findNearNeighbors(simMat, ...
      size(simMat,2), 'similarity');
    neighborIdxPrelim = adjustSelfSimilarityRank(neighborIdxPrelim);
    neighborIdxPrelim(:,1) = [];
  else
    error(['Unsupported neighbrhood type: ' neighborType]);
  end
  
  
% Iterate through all the "test" samples in this cross-validation fold
for iTest = 1:nTests

  thisTestSampleIdx = testIdx(iTest);
  candidateNeighbors = neighborIdxPrelim(thisTestSampleIdx,:);
  
  % Remove the other test samples from this sample's possible
  % neighbors: this keeps the tests completely disjoint
  for jTest = 1:nTests
    idx = find( candidateNeighbors == testIdx(jTest) );
    candidateNeighbors(idx) = [];
  end % for jTest
  
  % Limit the search for neighbors to the pruned version of the candidate
  % neighbors.
  subSimVec = simMat(testIdx(iTest), candidateNeighbors);
  subClassLabels = classLabels(candidateNeighbors);
  subMat = simMat(:, candidateNeighbors);
  subMat = subMat(candidateNeighbors,:);

  % Second and final pass at finding the neighbors, after pruning out the
  % other test samples
  if strcmpi(regularizingParams.neighborType, 'kPerClass')
    neighborIdx = findNearNeighborsByClass(subSimVec, length(subSimVec), ...
      subClassLabels, 'similarity');
    neighborsToPick = neighborIdx{1};
    neighborsToPick = neighborsToPick(:, 1:regularizingParams.nNeighbors);
    neighborsToPick = neighborsToPick(:);
  else
    neighborIdx = findNearNeighbors(subSimVec, length(subSimVec), 'similarity');
    neighborsToPick = neighborIdx(:, 1:regularizingParams.nNeighbors);
    neighborsToPick = neighborsToPick(:);
  end
   
  % Define the actual neighborhood sim matrix, class labels and test
  % similarity vector to be used for classification.
  neighborClassLabels = subClassLabels(neighborsToPick);
  testPointSim = subSimVec( neighborsToPick );
  neighborSimMat = subMat( neighborsToPick, : );
  neighborSimMat = neighborSimMat( :, neighborsToPick );

  % Classify!
  [classEst{iTest}, posteriors{iTest} nonRegModelParams{iTest}] = ...
    classifyRegularizedLocalSda( testPointSim, neighborSimMat, ...
    neighborClassLabels, regularizingParams);
  
  errCount.avgMarginal = errCount.avgMarginal + ...
    double(classEst{iTest}.avgMarginal ~= trueClass(iTest));
  errCount.avgMarginalUniformPriors = errCount.avgMarginalUniformPriors + ...
    double(classEst{iTest}.avgMarginalUniformPriors ~= trueClass(iTest));
  errCount.avgJoint = errCount.avgJoint +  ...
    double(classEst{iTest}.avgJoint ~= trueClass(iTest));
  errCount.avgJointUniformPriors = errCount.avgJointUniformPriors +  ...
    double(classEst{iTest}.avgJointUniformPriors ~= trueClass(iTest));
  errCount.avgSimToCentroids = errCount.avgSimToCentroids +  ...
    double(classEst{iTest}.avgSimToCentroids ~= trueClass(iTest));
  errCount.avgAcrossClasses = errCount.avgAcrossClasses + ...
     double(classEst{iTest}.avgAcrossClasses ~= trueClass(iTest));
   
end %for iTest

return