function [errCount, classEst, posteriors, modelParams] = ... 
  evaluateTestsLocalBda(neighborhoodSize, neighborType, simMat, ...
  featureMat, testIdx, classLabels)
%evaluateTestsLocalBda
% REDO HELP
% Compute the local BDA classification results for the test splits.
% This function is designed to be used in conjunction with
% readPartitionFile() and with the partition files from the Similarity
% Learning Repository.
%
% Assumes dissimilarity data are used (not similarity).
%
% USAGE
% [errorCount, classEst, posteriors, modelParams] = evaluateTestsLocalBda( ...
%     neighborhoodSize, neighborType, simMat, featureMat, testIdx, ...
%     classLabels)
% neighborhoodSize: the size of the neighborhood to use.
% simMat: N-by-N matrix of pairwise dissimilarities (Euclidean distances
%     between feature vectors).
% featureMat: N-by-F matrix of feature vectors. The i-th row represents the
%     i-th sample described by F features and corresponds to the i-th
%     sample in simMat. 
% testIdx: vector of indices into the rows of featureMat and simMat
%    denoting the samples to be tested. It is a row from testIdxMat as
%    returned by readPartitionFile().
% classLabels: N-by-1 vector of class labels for each row of featureMat.
% errorCount: number of misclassified test samples.
% classEst: M-by-1 cell array; each element contains a structure as
%    returned by classifyRegularizedLocalSda(). 
% posteriors:  M-by-1 cell array; each element contains a structure as
%    returned by classifyRegularizedLocalSda(). 
% nonRegModelParams: M-by-1 cell array; each element contains a structure as
%    returned by classifyRegularizedLocalSda(). 
%
% See also readPartitionFile, classifyLocalBda.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, Jan 2009.
%$Id:$
%$DateTime:$
%$Author:$

nTests = length(testIdx);
classEst = zeros(nTests,1);
posteriors = cell(nTests,1);
modelParams = cell(nTests, 1);
errCount = 0;

% Iterate through all the test samples
for iTest = 1:nTests

  thisTestSampleIdx = testIdx(iTest);
  thisTestSampleFeatureVec = featureMat(thisTestSampleIdx,:); 
  thisTestSampleClass = classLabels(thisTestSampleIdx);
  
  % First pass at defining neighborhood: get as many neighbors from each
  % class as possible, making sure the self-similar sample is taken out of
  % the pool of candidate neighbors.
  if strcmpi(neighborType, 'kPerClass')
    neighborIdxPrelim = findNearNeighborsByClass(simMat(thisTestSampleIdx,:), ...
      size(simMat,2), classLabels, 'dissimilarity');
    neighborIdxPrelim = adjustSelfSimilarityRankByClass(neighborIdxPrelim, ...
      classLabels, 'remove');
      
    % Pull out the test sample's neighbors
    candidateNeighbors = neighborIdxPrelim{1};
    candidateNeighbors = candidateNeighbors(:);
  
  elseif strcmpi(neighborType, 'kTotal')
    neighborIdxPrelim = findNearNeighbors(simMat(thisTestSampleIdx,:), ...
      size(simMat,2), 'dissimilarity');
    neighborIdxPrelim = adjustSelfSimilarityRank(neighborIdxPrelim);
    neighborIdxPrelim(1) = [];
    candidateNeighbors = neighborIdxPrelim(:);
  else
    error(['Unsupported neighborhood type: ' neighborType]);
  end
  
  % Remove the other test samples from this sample's possible
  % neighbors: this keeps the tests completely disjoint
  for jTest = 1:nTests
    idx = find( candidateNeighbors == testIdx(jTest) );
    candidateNeighbors(idx) = [];
  end % for jTest
  
  % Limit the search for neighbors to the pruned version of the candidate
  % neighbors.
  subSimVec = simMat(testIdx(iTest), candidateNeighbors);
  subClassLabels = classLabels(candidateNeighbors);
  subFeatureMat = featureMat(candidateNeighbors,:);
  
  % Second and final pass at finding the neighbors, after pruning out the
  % other test samples
  if strcmpi(neighborType, 'kPerClass')
    neighborIdx = findNearNeighborsByClass(subSimVec, length(subSimVec), ...
      subClassLabels, 'dissimilarity');
    neighborsToPick = neighborIdx{1};
    neighborsToPick = neighborsToPick(:, 1:neighborhoodSize);
    neighborsToPick = neighborsToPick(:);
  else
    neighborIdx = findNearNeighbors(subSimVec, length(subSimVec), 'dissimilarity');
    neighborsToPick = neighborIdx(:, 1:neighborhoodSize);
    neighborsToPick = neighborsToPick(:);
  end
   
  % Define the actual neighborhood features matrix, class labels and test
  % similarity vector to be used for classification.
  neighborClassLabels = subClassLabels(neighborsToPick);
  neighborFeatureMat = subFeatureMat(neighborsToPick,:);
  
  % Classify!
  [classEst(iTest), posteriors{iTest} modelParams{iTest}] = ...
    classifyLocalBda( thisTestSampleFeatureVec, neighborFeatureMat, ...
    neighborClassLabels);
  
  errCount = errCount + double(classEst(iTest) ~= thisTestSampleClass );

end %for iTest

return