function [classEst, logPosteriors, modelParams] = classifyLocalSda(...
  testPointSim, neighborSimMat, neighborClassLabels, centroidType, ...
  varargin)
%classifyLocalSda - classify a test point with local SDA.
% Classifies a test point based on its similarity to a set of other points
% using similarity discriminant analysis (SDA).
%
% USAGE
% [classEst, logPosteriors, modelParams] = classifyLocalSda( ...
% simVec, simMat, classLabels, centroidType, simDomain)
% simVec: 1-by-M vector of test point similarities to each of the set points.
% simMat: M-by-M matrix of pairwise similarities of all set points
% classLabels: M-by-1 vector of class labels of the points in the set.
% centroidType - string denoting the type of centroid to use. See
%     findCentroid() for a list of allowed centroid types.
% simDomain - vector of all possible similarity values. Optional parameter;
%     if it is not provided, it is constructed from simMat.
% classEst - class estimate for the test point corresponding to simVec.
% logPosteriors - log of the posterior probabilites for the classes.
% modelParams - structure with the computed maximum entropy model
%     parameters used for classification. See computeMaxEntModels for
%     details.
%
% See also classifySda

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of washington, November 2006.
%$Id:$
%$DateTime:$
%$Author:$

uniqueClassLabels = unique(neighborClassLabels);
nClasses = length(uniqueClassLabels);
logPosteriors = -Inf * ones(1, nClasses);
modelParams = initSdaModels;
modelParams.classLabels = uniqueClassLabels;

%% All neighbors are from one class --> classify as that class; done.
if nClasses == 1
  classEst = uniqueClassLabels;
  logPosteriors = 0;
  modelParams.classPriors = 1;
  modelParams.classCounts = size(neighborSimMat,1);
  %The other fields are empty
  return
end

%% A class has only one neighbor --> classify using nearest-centroid; done.
nSamplesPerClass = hist(neighborClassLabels, uniqueClassLabels);
if any(nSamplesPerClass == 1)
  [classEst, modelParams] = classifyLocalNearestCentroid( ...
    testPointSim, neighborSimMat, neighborClassLabels, centroidType);
  
  % For consistency, add these empty fields
  modelParams.gamma = [];
  modelParams.lambda = [];
  modelParams.meanSim = [];
  modelParams.estErr = [];
  modelParams.simDomain = [];
  return

end % if any(nSamplesPerClass == 1)


%% All other cases --> classify based on MaxEnt models
simDomain = local_determineSimDomain(testPointSim, neighborSimMat, ...
  neighborClassLabels, centroidType, varargin{:});
modelParams = computeMaxEntModels(neighborSimMat, neighborClassLabels, ...
  centroidType, simDomain);

% Classify
[classEst, logPosteriors] = classifySda(testPointSim, modelParams);

return %END main function



%% LOCAL FUNCTIONS
% local_determineSimDomain()
function simDomain = local_determineSimDomain(testPointSim, ...
  neighborSimMat, neighborClassLabels, centroidType, varargin)
% The similarity domain consists of all the similarity values observed in
% the training set plus the similarity of the test point to the classes;
% however user can override this definition of similarity domain by
% providing a similarity domain vector as an argument.
switch nargin
  case 5
    simDomain = varargin{:};
  case 4
    centroidModel = computeCentroidModels(neighborSimMat, ...
      neighborClassLabels, centroidType);
    simToCentroids = testPointSim(:, centroidModel.centroidIdx);
    simDomain = unique(unique(neighborSimMat));
    simDomain = [simDomain(:); simToCentroids(:)];
    simDomain = unique(simDomain);
  otherwise
    error('Invalid number of arguments');
end %switch nargin

return %END local_determineSimDomain()