function [classEstStruct, posteriorsStruct, nonRegModelParams] = ...
  classifyRegularizedLocalSda(testPointSim, neighborSimMat, ...
  neighborClassLabels, regularizingParams)
%classifyRegularizedLocalSda - classify a test point with regularized local SDA.
% Classifies a test point based on its similarity to a set of other points
% using regularized similarity discriminant analysis (SDA).
% USAGE
% [classEst, probs, modelParams] = classifyRegularizedLocalSda( ...
%    simVec, simMat, classLabels, regularizingParams)
% simVec: 1-by-M vector of test point similarities to each of the set points.
% simMat: M-by-M matrix of pairwise similarities of all set points
% classLabels: M-by-1 vector of class labels of the points in the set.
% regularizingParams: a structure containint the parameter necessary for
%    regularization, as returned by the function
%    createRegularizingParamsStruct().
% classEst - structure containing the class estimates for the test point.
%     Each filed corresponds to regularizaing in a slightly different way:
%     * avgMarginal: average of local class-conditional marginal pmfs 
%     * avgJoint: average of local class-conditional joint pmfs.
%     * avgAcrossClasses = DEPRECATED
%     * avgMarginalUniformPriors = DEPRECATED
%     * avgJointUniformPriors = DEPRECATED
%     * avgSimToCentroids: DEPRECATED
% probs - structure of posterior probabilites for the classes. The fields
%     correspond to the fields of classEst
% modelParams - structure with the computed maximum entropy model
%     parameters for the non-regularized local SDA models, as returned by
%     computeMaxEntModels.
%
% See also classifyLocalSda, createRegularizingParamsStruct,
% computeAverageLocaSdaModels, computeMaxEntModels.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, January 2009..
%$Id:$
%$DateTime:$
%$Author:$


%% Case in which the regularization parameter is 0 --> reduces to local SDA
if regularizingParams.regParam == 0
  [classEst, logPosteriors, nonRegModelParams] = classifyLocalSda( ...
    testPointSim, neighborSimMat, neighborClassLabels, ...
    regularizingParams.centroidType, regularizingParams.simDomain);
  classEstStruct.avgMarginal = classEst;
  classEstStruct.avgMarginalUniformPriors = classEst;
  classEstStruct.avgSimToCentroids = classEst;
  classEstStruct.avgJoint = classEst;
  classEstStruct.avgJointUniformPriors = classEst;
  classEstStruct.avgAcrossClasses = classEst;
  posteriors = exp(logPosteriors);
  posteriorsStruct.avgMarginal = posteriors;
  posteriorsStruct.avgMarginalUniformPriors = posteriors;
  posteriorsStruct.avgSimToCentroids = posteriors;
  posteriorsStruct.avgJoint = posteriors;
  posteriorsStruct.avgJointUniformPriors = posteriors;
  posteriorsStruct.avgAcrossClasses = posteriors;
  return
end 


%% Compute non-regularized MaxEnt models and test point probabilities
simDomain = local_determineSimDomain(testPointSim, neighborSimMat, ...
  neighborClassLabels, regularizingParams.centroidType, regularizingParams.simDomain);

nonRegModelParams = computeMaxEntModels(neighborSimMat, neighborClassLabels, ...
  regularizingParams.centroidType, simDomain);
testToCentroidSimVec = testPointSim(nonRegModelParams.centroidIdx);

nClasses = length(regularizingParams.classLabels);
nLocalClasses = length(nonRegModelParams.classLabels);


% Apply parameter correction if necessary
tempLambda = nonRegModelParams.lambda;
tempGamma = nonRegModelParams.gamma;
[tempLambda, tempGamma] = applyMaxEntParameterCorrection(tempLambda, ...
  tempGamma, nonRegModelParams.meanSim, testToCentroidSimVec);
tCMat = repmat(testToCentroidSimVec, nLocalClasses,1);
% CAREFUL: tempLambda could contain -Inf and tCMat could contain 0, and 
% -Inf * 0 = NaN. This affects the classification because
% the regularizing probability is overwhelmed by the NaN here. What we want
% is % -Inf * 0 = -Inf, because to us -Inf signifies wanting to zero out
% the probability of whatever similarity value we are looking at. We correct
% this below.
% Also note: in non-regularized local SDA this problem does not matter
% because NaNs are ingored by the max() operation, which seeks the class
% with the maximum posterior probability, so the classification outcome is
% unaffected. LGC, Jan 21, 2009. See also classifySda()
exponentMat = (tempLambda .* tCMat);
nanProblemIdx = isnan(exponentMat); % NaN correction
exponentMat(nanProblemIdx) = -Inf; % NaN correction
classProbsMat = tempGamma .* exp(exponentMat);
classProbs = prod(classProbsMat,2);

%% Pick out the appropriate probabilities from the regularizing pmfs

% Find index into similarity domain - assumes discrete similarity and that
% similarities being tested appear in simDomain
nClasses = length(regularizingParams.classLabels);
nLocalClasses = length(nonRegModelParams.classLabels);
% NOTE for kPerClass neighborhood type, nLocalClasses = nClasses
simIdx = zeros(1, nLocalClasses);
for hClass = 1:nLocalClasses
  simIdx(hClass) = find(regularizingParams.simDomain == ...
    testToCentroidSimVec(hClass));
end

%% Regularize using different strategies
alpha = regularizingParams.regParam;

% Regularization using the average pmfs of the class-conditional marginals
% This is the first implementation we used 
avgMarginalRegularizingProbs = -1e6 *ones(nLocalClasses, nLocalClasses);
for gClass = 1:nLocalClasses
   for hClass = 1:nLocalClasses
     thisProbVec = ...
       regularizingParams.avgMarginalConditionalLocalSdaPmfs{gClass, hClass};
     avgMarginalRegularizingProbs(gClass, hClass) = thisProbVec(simIdx(hClass)); 
   end % for hClass
end % for gClass
avgMarginalRegularizingClassProbs = prod(avgMarginalRegularizingProbs,2);

for gClass = 1:nLocalClasses
  classIdx(gClass) = find( regularizingParams.classLabels == ...
    nonRegModelParams.classLabels(gClass) );
end
regularizingPriors = regularizingParams.classPriors(classIdx);
regularizingPriors = regularizingPriors(:);

temp_localPosteriors = (1 - alpha) *  ...
  classProbs(:) .* nonRegModelParams.classPriors(:);

avgMarginalPosteriors = temp_localPosteriors + ...
  alpha * avgMarginalRegularizingClassProbs(:) .* regularizingPriors;

[maxAvgMarginalPosterior, classEstIdx] = max(avgMarginalPosteriors);
avgMarginalClassEst = nonRegModelParams.classLabels(classEstIdx);

% Variant of above, but using uniform priors - DEPRECATED
avgMarginalPosteriors_uniformPriors = (1 - alpha) * classProbs(:) + ...
  alpha * avgMarginalRegularizingClassProbs(:);
[maxAveMarginalPosterior_uniformPriors, classEstIdx] = ...
  max(avgMarginalPosteriors_uniformPriors);
avgMarginalClassEst_uniformPriors = nonRegModelParams.classLabels(classEstIdx);

% Regularizing using the average pmfs of class-conditional marginals BUT
% regularizing the marginals first and then combining into the joint probs
% and then posteriors
avgMarginalProbMat = (1 - alpha) * classProbsMat + ...
  alpha * avgMarginalRegularizingProbs;
weightedPriors = (1 - alpha) * nonRegModelParams.classPriors(:) + ...
  alpha * regularizingPriors(:);
avgJointProbs = prod(avgMarginalProbMat,2) .* weightedPriors(:);

% Variant of above, but using uniform priors - DEPRECATED
avgJointPosteriors_uniformPriors = avgJointProbs;
[maxAvgJointPosterior_uniformPriors, classEstIdx] = ...
  max(avgJointPosteriors_uniformPriors);
avgJointClassEst_uniformPriors = nonRegModelParams.classLabels(classEstIdx);

avgJointPosteriors = avgJointProbs(:) .* regularizingPriors(:);
[maxAvgJointPosterior, classEstIdx] = ...
  max(avgJointPosteriors);
avgJointClassEst = nonRegModelParams.classLabels(classEstIdx);

% Regularizing with the average pmf across classes - NOT THAT USEFUL
regProbVec = -1e6 * ones(1, nLocalClasses);
for hClass = 1:nLocalClasses
  tmpVec = regularizingParams.avgAcrossClasses{classIdx(hClass)};
  regProbVec(hClass) = tmpVec(simIdx(hClass));
end
avgAcrossClassesProbs = (1 - alpha) * classProbsMat + ...
  alpha * repmat(regProbVec, nLocalClasses, 1);
avgAcrossClassesPosteriors = prod(avgAcrossClassesProbs, 2) .* ...
  regularizingPriors(:);
[maxAvgAcrossClassesPosterior, classEstIdx] = ...
  max(avgAcrossClassesPosteriors);
avgAcrossClassesClassEst = nonRegModelParams.classLabels(classEstIdx);

% Regularization using the average unconditional sim to class centroid pmf
% This is not really a regularization: it's a weighted product
% DEPRECATED, OFTEN WORSE THAN NON-REG LOCAL SDA
avgSimToCentroidProbs = 1e-6 * ones(1,nLocalClasses);
for hClass = 1:nLocalClasses
  avgSimToCentroidProbs(hClass) = ...
    regularizingParams.avgSimToCentroidHistPmfs(simIdx(hClass),classIdx(hClass));
end % for hClass
avgSimToCentroidPosteriors = ...
  classProbsMat .* repmat(avgSimToCentroidProbs(:).', nLocalClasses,1);
avgSimToCentroidPosteriors = prod(avgSimToCentroidPosteriors,2);
[maxAvgSimToCentroidPosteriors, classEstIdx] = ...
  max(avgSimToCentroidPosteriors);
avgSimToCentroidsClassEst = nonRegModelParams.classLabels(classEstIdx);

% TO DO
% Regularize using the average Bayes local models

%% Put results in a structure 
classEstStruct.avgMarginal = avgMarginalClassEst;
classEstStruct.avgJoint = avgJointClassEst;
classEstStruct.avgAcrossClasses = avgAcrossClassesClassEst; % NOT THAT GREAT

% THESE THREE ARE DEPRECATED
classEstStruct.avgMarginalUniformPriors = avgMarginalClassEst_uniformPriors;
classEstStruct.avgJointUniformPriors = avgJointClassEst_uniformPriors;
classEstStruct.avgSimToCentroids = avgSimToCentroidsClassEst;


posteriorsStruct.avgMarginal = avgMarginalPosteriors;
posteriorsStruct.avgJoint = avgJointPosteriors;

posteriorsStruct.avgMarginalUniformPriors = avgMarginalPosteriors_uniformPriors;
posteriorsStruct.avgJointUniformPriors = avgJointPosteriors_uniformPriors;
posteriorsStruct.avgSimToCentroid = avgSimToCentroidPosteriors;
posteriorsStruct.avgAcrossClasses = avgAcrossClassesPosteriors;

%NOTE: I named the quantities incorrectly: avgMarginal is actually the joint
%and vice-versa. LGC, Jan 28, 2009

return % END of classifyRegularizedLocalSda()


%% LOCAL FUNCTIONS
% local_determineSimDomain()
function simDomain = local_determineSimDomain(testPointSim, ...
  neighborSimMat, neighborClassLabels, centroidType, varargin)
% The similarity domain consists of all the similarity values observed in
% the training set plus the similarity of the test point to the classes;
% however user can override this definition of similarity domain by
% providing a similarity domain vector as an argument.
switch nargin
  case 5
    simDomain = varargin{:};
  case 4
    centroidModel = computeCentroidModels(neighborSimMat, ...
      neighborClassLabels, centroidType);
    simToCentroids = testPointSim(:, centroidModel.centroidIdx);
    simDomain = unique(unique(neighborSimMat));
    simDomain = [simDomain(:); simToCentroids(:)];
    simDomain = unique(simDomain);
  otherwise
    error('Invalid number of arguments');
end %switch nargin

return %END local_determineSimDomain()

% local_checkModelConsistency()
% TO DO
%function out = local_checkModelConsistency()
% What wo check:
% number of classes
% similarity domain
% 
  

  
  



