function [averageModelParams, localSdaModelParams] = ...
  computeDirichletLocalSdaModels(simMat, classLabels, centroidType, ...
  neighborhoodSizeVec, varargin)
%computeDirichletLocalSdaModels - compute Dirichlet local SDA models.
% REDO THE HELP
% Computes the average of local SDA models for various neighborhood sizes.
% The average is over N samples, where N is the dimension of the input
% square similarity matrix. Each local SDA model is computed by leaving one
% sample out and computing its local SDA model parameters for a fixed
% neighborhood size from the remaining N-1 samples. Several average local models
% are computed, one for each desired neighborhood size. The neighborhood 
% type is specified as an input, it can be either k neighbors per class or
% k total neighbors. 
% USAGE
% [averageModelParams localSdaModelParams] = ...
%     computeAverageLocalSdaModels(simMat, classLabels, centroidType, ...
%     neighborhoodSizeVec, simDomain, neighborType)
%
% simMat - N-by-N matrix of pairwise similarities.
% classLabels - length-N vector of class labels corresponding to the
%     columns of simMat.
% centroidType - string denoting the type of centroid to use. Only
%     'maxSumSim' has been tested. 
% neighborhoodSizeVec - vector of neighborhood sizes to use for the average
%     models.
% simDomain - optional vector of possible similarity values.
% neighborType - optional, either 'KPerClass' or 'kTotal' (default).
% averageModelParams - structure containing the parameters for the average
%    local models. The fields are:
%    * avgMarginalConditionalLocalSdaPmfs: C-by-C-by-K cell array containing
%    the average local SDA pmfs, where C is the number of classes and K is
%    the neighborhood size. The {g,h,k} cell element is a probability mass
%    function vector for the similarity of sample x to the centroid of
%    class h, given that sample x is from class g. The pmfs are over the
%    the values in simDomain.
%    * neighborhoodSize: vector of neighborhood sizes used. This may be
%    different from neighborhoodSizeVec due to the size of the data.
%    * simDomain: vector of possible similarity values.
%    * centroidType: string denoting the type of centroid used, set to
%        'maxsumSim'.
%    * regType: string denoting the regularization used, set to
%        'averageLocalSda'.
%    * neighborType: string denoting the type of neighborhood used.
%    * classPriors: the class prior probabilities (Laplace correction applied)
%    * classCounts: number of data samples from each class. 
%    * classLabels: the possible class labeles.
%
% See also classifyRegularizedLocalSda.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, September 2009.

%% Process input arguments 

% Similarity domain
switch nargin
  case 4
    simDomain = unique(simMat(:));
  case 5 
    simDomain = unique(varargin{1});
  otherwise
    error('Incorrect number of inputs');
end % swith nargin

% Neighborhood sizes to consider
neighborhoodSizeVec = unique(neighborhoodSizeVec);
nSizes = length(neighborhoodSizeVec);


