function modelParams = computeMixtureMaxEntModels( simMat, classLabels, ...
  modelParams, varargin)
%computeMixtureMaxEntModels - compute maximum entropy mixture model parameters. 
% Computes the parameters for mixtures of mean similarity-constrained maximum
% entropy models for the two-class case. Calls computeLambda() to
% compute the maximum entropy model parameters for each of the similarity
% models. Only two-class problems are supported. 
%
% USAGE
% modelParams = computeMixtureMaxEntModels(simMat, classLabels, ...
%     centroidIdx, simDomain)
% siMat: N-by-N matrix of pairwise similarities.
% classLabels: N-by-1 vactor of class labels corresponding to each row of
%     simMat. Must be exactly 2 unique classes.
% modelParams: structure containing the model parameters. On input it
%     consists of either previously-computed models to be updated or
%     initial bare-bones models computed by computeMultiCentroidModels(). 
%     The fields on output are:
%     * isTrivial: boolean flag, true if only one class is represented in
%     the training data.
%     * classPriors: C-by-1 vector of class prior probabilities, where C is
%     the number of classes represented in classLabels. 
%     * classCounts: C-by-1 vector of class frequency counts for the
%     training set.
%     * classLabels: C-by-1 vector of unique class labels.
%     * centroidIdx: 1-by-M vector containing the indices into the
%     columns of simMat corresponding to the M class centroids.
%     * centroidType: string denoting the type of centroid used for the
%     models.
%     * lambda: 2-by-1 cell array of matrices, each element of the matrix is an
%     exponent in maximum entropy distribution. The parameters are
%     organized so that lambda{1} is a matrix containing mixture SDA model
%     parameters for class 1: 
%     row 1 - model parameters for class 1 centroids conditioned on data
%     being in class 1.
%     row 2 - model parameters for class 1 centroids conditioned in data
%     being in class 2.
%     Similarly, lambda{2} is a matrix containing multi-centroid model
%     parameters for class 2.
%     * gamma: cell array of matrices organized as in the same way as
%     lambda.
%     * meanSim: cell array of matrices of mean similarity values,
%     organized as lambda.
%     * estErr: cell array of matrices of etimation errors, organized as
%     lambda. 
%     * simDomain: vector of possible similarity values.
% simDomain: optional vector of possible similarity values. If not
%     provided, it is computed by taking the unique values of simMat. 
%
% See also computeMixtureSdaModels, computeMultiCentroidModels, classifyMixtureSda.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, March 2007.
%$Id:$
%$DateTime:$
%$Author:$

%% Quick check on number of classes
uniqueClassLabels = unique(classLabels);
nClasses = length(uniqueClassLabels);
if nClasses ~= 2
  error(['Found ' num2str(nClasses) ' classes; only 2 classes allowed.']);
end

%% Get similarity domain vector
% Is the similarity domain provided or should it be inferred from the
% similarity matrix?
switch nargin
  case 4 % provided 
    similarityDomainVector = varargin{1};
  case 3 % compute
    similarityDomainVector = unique(unique(simMat));
  otherwise
    error('Wrong number of input arguments');
end % case


%% Compute MaxEnt model parameters: lambdas, gammas, etc. ...
% Each lambda and gamma matrix in the cell array contains model parameters 
% organized as follows:
% lambdaCell{1} is a matrix containing mixture SDA model parameters
% for class 1: 
%     row 1 - model parameters for class 1 centroids conditioned on data
%     being in class 1.
%     row 2 - model parameters for class 1 centroids conditioned on data
%     being in class 2.
% Similarly, lambdaCell{2} is a matrix containing multi-centroid model
% parameters for class 2:
%     row 1 - model parameters for class 2 centroids conditioned on data
%     being in class 1.
%     row 2 - model parameters for class 2 centroids conditioned on data
%     being in class 2.
% 
% Pre-allocate
lambdaCell = cell(nClasses,1);
gammaCell = cell(nClasses,1);
estErrCell = cell(nClasses,1);
meanSimCell = cell(nClasses,1);
  
for iCentroidClass = 1:nClasses
  thisClassCentroidIdx = ( modelParams.centroidClassLabels == ...
    modelParams.classLabels(iCentroidClass) );
  thisClassCentroidIdx = modelParams.centroidIdx(thisClassCentroidIdx);
  thisClassNumCentroids = length(thisClassCentroidIdx);
 
  % Initialize to large value for easier debugging
  lambdaMat = 10e10 * ones(nClasses, thisClassNumCentroids);
  gammaMat = 10e10 * ones(nClasses, thisClassNumCentroids);
  estErrMat = 10e10 * ones(nClasses, thisClassNumCentroids);
  meanSimMat = 10e10 * ones(nClasses, thisClassNumCentroids);
  
  for iClass = 1:nClasses
    thisClassIdx = classLabels == modelParams.classLabels(iClass);
    simToCentroids = simMat(thisClassIdx, thisClassCentroidIdx);
    
    % Are we updating existing lambda and gamma or computing them for the
    % first time?
    if all(isfield(modelParams, {'lambda', 'gamma'})) %Updating
      thisLambdaCell = modelParams.lambda{iCentroidClass};
      thisLambda = thisLambdaCell(iClass,:);
      thisGammaCell = modelParams.gamma{iCentroidClass};
      thisGamma = thisGammaCell(iClass,:);
       thisComponentWeights = ...
         modelParams.centroidWeights{iCentroidClass}(iClass,:);
      thisComponentPriors = ...
        modelParams.componentPriors{iCentroidClass}(iClass,:);
      meanSim = local_computeWeightedAverageSimilarity(simToCentroids, ...
        thisLambda, thisGamma, thisComponentWeights, thisComponentPriors);
    else % First-time computation
      meanSim = mean(simToCentroids,1); 
    end
    
    %Pre-allocate
    lambda = zeros(1, thisClassNumCentroids);
    gamma = zeros(1, thisClassNumCentroids);
    estErr = zeros(1, thisClassNumCentroids);
    
    % This loop computes model parameters for each centroid in the current
    % class being considered by the outer loop
    for iCentroid = 1:thisClassNumCentroids
      [lambda(iCentroid), gamma(iCentroid), estErr(iCentroid)] = ...
        computeLambda(meanSim(iCentroid), similarityDomainVector);
    end %for iCentroid
    lambdaMat(iClass, :) = lambda;
    gammaMat(iClass, :) = gamma;
    estErrMat(iClass, :) = estErr;
    meanSimMat(iClass,:) = meanSim;

  end % for iClass

  lambdaCell{iCentroidClass} = lambdaMat;
  gammaCell{iCentroidClass} = gammaMat;
  estErrCell{iCentroidClass} = estErrMat;
  meanSimCell{iCentroidClass} = meanSimMat;
  
end% for iCentroidClass
  
%% Put results in structure
% Simply augment multi centroid parameter structures with MaxEnt model
% parameters
modelParams.isTrivial = false;

modelParams.lambda = lambdaCell;
modelParams.gamma = gammaCell;
modelParams.meanSim = meanSimCell;
modelParams.estErr = estErrCell;

% Keep ancillary information too.
modelParams.simDomain = similarityDomainVector;

return

%% LOCAL FUNCTIONS
%local_computeWeightedAverageSimilarity()
% Computes the weighted average mean similarity. A point's weight is
% the normalized probability that is was generated by a particular
% mixture component. 
function weightedAverageSim = local_computeWeightedAverageSimilarity( ...
  simToCentroids, lambda, gamma, componentWeights, componentPriors)

% Make sure this is a row vector
componentPriors = componentPriors(:).';
componentWeights = componentWeights(:).';

% NOTE: component weights do not appear explicitly in the formula below;
% they are implicitly taken into account by the scaling factor in the
% reponsibilities. See my dissertation. LGC.

[nData, nCentroids] = size(simToCentroids);
lambdaMat = repmat(lambda, nData,1);
gammaMat = repmat(gamma, nData, 1);
probValues = gammaMat .* exp(lambdaMat .* simToCentroids);
probValues = probValues .* repmat(componentPriors, nData, 1);
probValues = probValues .* repmat(componentWeights, nData, 1);
probNormFactor =  sum(probValues, 2);
probNormFactor = repmat(probNormFactor, 1, nCentroids);
responsibilities = probValues ./ probNormFactor;
respNormFactor = sum(responsibilities);
respNormFactor = repmat(respNormFactor, nData,1);
normalizedResponsibilities = responsibilities ./ respNormFactor;
weightedAverageSim = sum(simToCentroids .* normalizedResponsibilities, 1);

return