function neighborIdxByClass = findNearNeighborsByClass(inMat, ...
  nNeighborsPerClass, classLabels, varargin)
%findNearNeighborsByClass - find k-nearest neighbors from each class.
% Finds the indices of a sample's nearest neighbors, given a matrix of
% pairwise (dis)similarities.
%
% USAGE 
% neighborIdxByClass = findNearNeighborsByClass(inMat, nNeighbors, ...
%     classLabels, simMode)
% inMat - N-by-M matrix of (dis)similarities; each row contains a sample's
%     (dis)similarities to M samples.
% nNeighbors - number of neighbors to return from each class.
% classLabels - vector of M class labels for the columns of inMat.
% simMode - optional, either 'similarity' (default) or 'dissimilarity',
%     depending on whether inMat contains similarities or dissimilarities.
% neighborIdxByClass - N-by-1 cell array; each cell contains a
%    C-by-nNeighbors matrix, where C is the number of unique classes
%    represented in classLabels. The i-th row contains the indices into the
%    columns of simMat corresponding to the nNeighbors-nearest neighbors
%    from the i-th class.
%
% See also adjustSelfSimilarityRankByClass.

% Written by Luca Cazzanti, engineer
% Applied Physics Laboratory - University of Washington, Jan. 2009.
%$Id:$
%$DateTime:$
%$Author:$

% Similarities or dissimilarities?...
switch nargin
  case 3 % default is similarities --> descend
    sortMode = 'descend';
  
  case 4
    switch varargin{1}
      case 'similarity'
        sortMode = 'descend';
      case 'dissimilarity'
        sortMode = 'ascend';
      otherwise
        error('Invalid similarity type');
    end % switch inMode
  
  otherwise
    error('Wrong number of inputs');
end % switch nargin

% How many samples are in each class?
uniqueClassLabels = unique(classLabels);
samplesPerClass = hist(classLabels, uniqueClassLabels);

% Can only pick out as many neighbors from each class as the number of
% samples in the smallest class.
maxNeighborsPerClass = min(samplesPerClass);
maxNeighborsPerClass = min([maxNeighborsPerClass nNeighborsPerClass]);

nClasses = length(uniqueClassLabels);
nSamples = size(inMat,1);
neighborIdxByClass = cell(nSamples,1);

% Sort similarities
[sortMat, sortIdx] = sort(inMat, 2, sortMode);

for iSample = 1:nSamples
  thisNeighborIdxMat = -1e6 * ones(nClasses, maxNeighborsPerClass);
  for iClass = 1:nClasses
    thisClassIdx = find(classLabels(sortIdx(iSample,:)) == ...
      uniqueClassLabels(iClass));
    thisNeighborIdxMat(iClass,:) = sortIdx(iSample, ...
      thisClassIdx(1:maxNeighborsPerClass));
  end
  neighborIdxByClass{iSample} = thisNeighborIdxMat;
end

% CAREFUL: might need to run adjustSelfSimilarityRankByClass() after
% running this function. We want to ensure sortIdx(rowIdx,1) = rowIdx, that
% is  each element is its own nearest neighbor even if it is equally
% similar to other elements of the row. Matlab's sort() does not do this;
% it maintains the original sort order if there are (dis)similarity ties so
% we might need to adjust the neighbors ranks. This is accomplished by the
% separate function adjustSelfSimilarityRankByClass().

return